/*
     This file is part of GNUnet
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/
/**
 * Wrapper around malloc/free.
 * @file include/util/xmalloc.h
 * @author Krista Bennett
 **/

#ifndef XMALLOC_H
#define XMALLOC_H

#include "config.h"

/**
 * Allocate memory. Checks the return value, aborts if no more
 * memory is available.
 * @deprecated, use MALLOC
 **/
void * xmalloc(int size, 
	       const char * site);

/**
 * Allocate memory. Checks the return value, aborts if no more
 * memory is available.
 **/
void * xmalloc_(int size, 
		const char * filename,
		const int linenumber);

#define MALLOC(size) xmalloc_(size, __FILE__,__LINE__)

/**
 * Free memory. Merely a wrapper for the case that we
 * want to keep track of allocations.
 * @deprecated, use FREE
 **/
void xfree(void * ptr, 
	   const char * site);

/**
 * Free memory. Merely a wrapper for the case that we
 * want to keep track of allocations.
 **/
void xfree_(void * ptr, 
	    const char * filename,
	    const int linenumber);

#define FREE(ptr) xfree_(ptr, __FILE__, __LINE__)

/**
 * Free if not NULL.
 * @deprecated, use FREENONNULL
 **/
void freeNonNull(void * ptr,
		 const char * site);

#define FREENONNULL(ptr) if (ptr != NULL) { FREE(ptr); } else {}

/**
 * Dup a string.
 **/
char * xstrdup_(const char * str,
		const char * filename,
		const int linenumber);

#define STRDUP(a) xstrdup_(a,__FILE__,__LINE__)

/**
 * Grow an array, the new elements are zeroed out.
 * Grows old by (*oldCount-newCount)*elementSize
 * bytes and sets *oldCount to newCount.
 * @param old address of the pointer to the array
 *        *old may be NULL
 * @param elementSize the size of the elements of the array
 * @param oldCount address of the number of elements in the *old array
 * @param newCount number of elements in the new array, may be 0 (then *old will be NULL afterwards)
 **/
void xgrow_(void ** old,
	    int elementSize,
	    int * oldCount,
	    int newCount,
	    char * filename,
	    const int linenumber);

#define GROW(arr,size,tsize) xgrow_((void**)&arr, sizeof(arr[0]), &size, tsize, __FILE__, __LINE__)

#endif
/* end of xmalloc.h */
